module advanceddelegate.demo;

private import std.stdio;
private import advanceddelegate.all;

/* This is a demo module for advanceddelegate */
int main(char[][] argv)
{
    /*  The most simple thing: define a new function "add"
        The definition looks a lot nicer with the "auto" keyword.
    */
    auto add = AdvancedFunction(function int(int a, int b) { return a+b; });

    /* Using these functions is pretty easy. */
    writefln("add(2,2): ", add.Eval(2,2));
    /* This is exactly the same as: */
    writefln("add(2,2): ", add(2,2).Eval);
    /* Or: */
    writefln("add(2,2): ", add(2)(2).Eval);

    /*  Now you can easily use partial application:
        When we supply "2" as the first parameter of the add function,
        we get a function that adds 2 to its single int argument.
    */
    auto add2 = add(2);

    writefln("add2(1): ", add2.Eval(1));
    writefln("add2(3): ", add2.Eval(3));

    /*  So, what was the nice thing about functional programming?
        Functions that take functions as their arguments! */
    auto applyToArray =
        AdvancedFunction
        (
            function int[](ADG1!(int, int) func, int[] array)
            {
                foreach(inout int x; array)
                    x = func.Eval(x);
                return array;
            }
        );

    /* We can now easily define a function that adds 2 to a whole array. */
    auto add2ToArray = applyToArray(add2);

    /* Test it... */
    static int[5] testarray = [1,7,8,2,9];
    writefln("add2ToArray([1,7,8,2,9]): ", add2ToArray(testarray).Eval);

    return 0;
}
